import camera
import network
import time
import urequests
from machine import Pin

# Dane Wi-Fi i serwera
SSID = 'ae'
PASSWORD = 'Xavlegbmao'
SERVER_URL = "http://192.168.43.220:5000"

latara = Pin(4,Pin.OUT)
latara.value(0)
# Funkcja łącząca się z Wi-Fi
def connect_wifi():
    wifi = network.WLAN(network.STA_IF)
    wifi.active(True)
    wifi.connect(SSID, PASSWORD)
    for _ in range(10):
        if wifi.isconnected():
            print("Connected to Wi-Fi.")
            return wifi
        time.sleep(1)
    print("Failed to connect to Wi-Fi.")
    return None
# Funkcja pobierająca macierz z serwera
def get_matrix_from_server():
    try:
        print("Fetching matrix from server...")
        response = urequests.post(f"{SERVER_URL}/get_matrix")
        if response.status_code == 200:
            data = response.json()
            print("Matrix received:", data['matrix'])
            return data['matrix']
        else:
            print("Error fetching matrix:", response.text)
    except Exception as e:
        print("Error fetching matrix:", e)
    return None

# Funkcja sprawdzająca, czy macierz jest "X"
def is_x(matrix):
    size = len(matrix)
    main_diagonal = [matrix[i][i] for i in range(size)]
    anti_diagonal = [matrix[i][size - i - 1] for i in range(size)]

    main_ones = sum(main_diagonal)
    anti_ones = sum(anti_diagonal)

    main_ratio = main_ones / size
    anti_ratio = anti_ones / size

    threshold = 0.7
    return main_ratio >= threshold and anti_ratio >= threshold

# Funkcja sprawdzająca, czy macierz to kwadrat
def is_square(matrix):
    size = len(matrix)
    tolerance = int(0.7 * size)  # Tolerancja na ilość 0 w obwódce (np. 20%)
    errors = 0

    # Sprawdzamy górną i dolną krawędź
    for i in range(size):
        if matrix[0][i] != 1: errors += 1
        if matrix[size - 1][i] != 1: errors += 1
    
    # Sprawdzamy lewą i prawą krawędź
    for i in range(size):
        if matrix[i][0] != 1: errors += 1
        if matrix[i][size - 1] != 1: errors += 1

    # Jeśli błędy mieszczą się w tolerancji, uznaj za kwadrat
    return errors <= tolerance

def is_triangle(matrix):
    size = len(matrix)

    # Sprawdzenie: punkt wierzchołka w górnym wierszu
    top_row = matrix[0]
    if sum(top_row) > 2:
        return False

    # Sprawdzenie: dwa dolne wiersze mają wypełnione punkty (ale mogą być nierówne)
    bottom_two_rows = matrix[-2:]
    for row in bottom_two_rows:
        if sum(row) < 2:  # Minimalnie dwa punkty w każdym z dolnych wierszy
            return False

    # Sprawdzenie: pozostałe wiersze tworzą kształt trójkąta
    for i in range(1, size - 2):  # Pomijamy pierwszy wiersz i dwa dolne
        if sum(matrix[i]) > i + 1:  # Każdy wiersz powinien mieć <= (i+1) punktów
            return False

    return True


def is_circle(matrix):
# Określamy punkty w krawędzi, które są częścią okręgu
    edge_points = [
        matrix[0][1], matrix[0][2], matrix[0][3], matrix[0][4],  
        matrix[1][4],  
        matrix[2][4],  
        matrix[3][4],  
        matrix[4][3], matrix[4][2], matrix[4][1], matrix[4][0], 
        matrix[3][0],  
        matrix[2][0],  
        matrix[1][0],  
    ]
    
    edge_points_count = sum(edge_points)
    if edge_points_count >= 8:
        if matrix[0][0] == 0 or matrix[0][4] == 0 or matrix[4][0] == 0 or matrix[4][4] == 0:
            return True
    return False

# Funkcja do wykrywania kształtu
def detect_shape(matrix):
    if is_x(matrix):
        return "X"
    elif is_square(matrix):
        return "Kwadrat"
    elif is_triangle(matrix):
        return "Trójkąt"
    elif is_circle(matrix):
        return "Koło"
    else:
        return "Inny kształt"

# Funkcja główna
def main():
    wifi = connect_wifi()
    if not wifi:
        return
    initialize_camera()

    while True:
         # Pobieranie macierzy i analiza
        matrix = get_matrix_from_server()
        if matrix:
            result = detect_shape(matrix)
            print(f"Detected shape: {result}")

        time.sleep(1)

if __name__ == "__main__":
    main()


