from flask import Flask, request, send_file
from PIL import Image
import numpy as np
import io

app = Flask(__name__)
latest_image = None  # Ostatni oryginalny obraz
processed_image = None  # Ostatni przetworzony obraz (binaryzowany)

@app.route('/upload', methods=['POST'])
def upload():
    global latest_image, processed_image
    try:
        # Odbierz obraz z ESP32
        if 'image' in request.files:
            file = request.files['image']
            image = Image.open(file)
        else:
            image = Image.open(io.BytesIO(request.data))

        # Zapisz oryginalny obraz
        latest_image = image.copy()

        # Przeskalowanie do 5x5 i konwersja do skali szarości
        resized_image = image.resize((5, 5)).convert('L')
        matrix = np.array(resized_image)

        # Binaryzacja (próg = 128)
        binary_matrix = (matrix < 120).astype(np.uint8)  # 1 dla czarnych, 0 dla białych

        # Przekształcenie macierzy na obraz (dla wizualizacji 0->255, 1->0)
        visual_matrix = (1 - binary_matrix) * 255  # Zamiana: 0=biały (255), 1=czarny (0)
        processed_image = Image.fromarray(visual_matrix.astype(np.uint8))

        log = f"Binaryzowana macierz (0=biały, 1=czarny):\n{binary_matrix}\n"
        return log, 200

    except Exception as e:
        return f"Błąd: {e}", 500

@app.route('/display', methods=['GET'])
def display():
    if latest_image:
        img_io = io.BytesIO()
        latest_image.save(img_io, 'JPEG')
        img_io.seek(0)
        return send_file(img_io, mimetype='image/jpeg')
    else:
        return "Nie ma obrazu do wyświetlenia", 404

@app.route('/display_processed', methods=['GET'])
def display_processed():
    if processed_image:
        img_io = io.BytesIO()
        processed_image.save(img_io, 'JPEG')
        img_io.seek(0)
        return send_file(img_io, mimetype='image/jpeg')
    else:
        return "Nie ma przetworzonego obrazu do wyświetlenia", 404

@app.route('/get_matrix', methods=['POST'])
def get_matrix():
    try:
        if latest_image:
            # Przeskalowanie do 5x5 i konwersja do skali szarości
            resized_image = latest_image.resize((5, 5)).convert('L')
            matrix = np.array(resized_image)

            # Binaryzacja (próg = 50)
            binary_matrix = (matrix < 120).astype(np.uint8)

            return {'matrix': binary_matrix.tolist()}, 200
        else:
            return {"error": "Brak obrazu do przetworzenia"}, 400
    except Exception as e:
        return f"Błąd: {e}", 500

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, debug=True)